/*   
 * Copyright 2007 Sun Microsystems, Inc.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */ 
package com.sun.syndication.propono.atom.server.impl;

import com.sun.syndication.propono.atom.common.AtomService;
import com.sun.syndication.propono.utils.Utilities;
import java.io.InputStream;
import java.util.Map;
import java.util.Properties;
import java.util.TreeMap;

/**
 * File based Atom service.
 * Supports one workspace per user.
 * Collections in workspace are defined in /propono.properties, for example:
 *
 * <pre>
 *    # Define list of collections to be offered
 *    propono.atomserver.filebased.collections=entries,gifimages
 *
 *    # Defines 'entries' collection, accepts entries
 *    propono.atomserver.filebased.collection.entries.title=Entries
 *    propono.atomserver.filebased.collection.entries.singular=entry
 *    propono.atomserver.filebased.collection.entries.plural=entries
 *    propono.atomserver.filebased.collection.entries.accept=entry
 *
 *    # Defines 'gifimages' collection, accepts only GIF files
 *    propono.atomserver.filebased.collection.gifimages.title=GIF Images
 *    propono.atomserver.filebased.collection.gifimages.singular=gif
 *    propono.atomserver.filebased.collection.gifimages.plural=gifs
 *    propono.atomserver.filebased.collection.gifimages.accept=image/gif
 * </pre>
 *
 * If no such properties are found, then service will fall back to two 
 * collections: 'entries' for Atom entries and 'resources' for any content-type.
 *
 * <p><b>URI structure used for accessing collections and entries</b></p>
 * 
 * <p>Collection feed (URI allows GET to get collection, POST to add to it)<br/>
 *     <code>[servlet-context-uri]/app/[workspace-handle]/[collection-plural] </code>
 * </p>
 *
 * <p>Collection entry (URI allows GET, PUT and DELETE)<br/>
 *     <code>[servlet-context-uri]/app/[workspace-handle]/[collection-singular]/entryid </code>
 * </p>
 *
 * <p>Collection entry media (URI allows GET, PUT and DELETE)<br/>
 *     <code>[servlet-context-uri]/app/[workspace-handle]/[collection-singular]/entryid/media/entryid</code>
 * </p>
 *
 * <p><b>Directory structure used to store collections and entries</b></p>
 *
 * <p>Collection feed (kept constantly up to date)<br/>
 *     <code>[servlet-context-dir]/[workspace-handle]/[collection-plural]/feed.xml</code> 
 * </p>
 *
 * <p>Collection entry (individual entries also stored as entry.xml files)<br/>
 *     <code>[servlet-context-dir]/[workspace-handle]/[collection-plural]/id/entry.xml</code>
 * </p>
 *
 * <p>Collection entry media (media file stored under entry directory)<br/>
 *     <code>[servlet-context-dir]/[workspace-handle]/[collection-plural]/id/media/id</code>
 * </p>
 */
public class FileBasedAtomService extends AtomService {
    private String baseDir = null;
    private Map workspaceMap = new TreeMap();
    private Map collectionMap = new TreeMap();
    private static Properties cacheProps = new Properties();
    private boolean firstTime = false;
    
    /** 
     * Creates a new instance of FileBasedAtomService.
     */
    public FileBasedAtomService(String userName, String baseDir, String appURI) throws Exception {
        this.baseDir = baseDir;
        String workspaceHandle = userName;

        // One workspace per user        
        FileBasedWorkspace workspace = new FileBasedWorkspace(workspaceHandle, baseDir);
        workspaceMap.put(userName, workspace);
                       
        if (firstTime) {
            synchronized(cacheProps) {
               InputStream is = getClass().getResourceAsStream("/propono.properties");
               cacheProps.load(is);
               firstTime = false;
            }
        }
        
        String colnames = cacheProps.getProperty("propono.atomserver.filebased.collections");
        if (colnames != null) {
            // collections specified in propono.properties, use those
            String[] colarray = Utilities.stringToStringArray(colnames,",");
            for (int i=0; i<colarray.length; i++) {
                String prefix = "propono.atomserver.filebased.collection." + colarray[i] + ".";
                String collectionTitle = cacheProps.getProperty(prefix + "title");
                String collectionSingular = cacheProps.getProperty(prefix + "singular");
                String collectionPlural = cacheProps.getProperty(prefix + "plural");
                String collectionAccept = cacheProps.getProperty(prefix + "accept");
                
                FileBasedCollection entries = new FileBasedCollection(
                        collectionTitle, workspaceHandle, 
                        collectionPlural, collectionSingular, 
                        baseDir, collectionAccept, appURI);
                workspace.addCollection(entries);
                // want to be able to look up collection by singular and plural names
                collectionMap.put(workspaceHandle + "|" + collectionSingular, entries);
                collectionMap.put(workspaceHandle + "|" + collectionPlural, entries);
            }
        } else {
            
            // Fallback to two collections. One collection for accepting entries 
            // and other collection for other ( resources/uploaded images etc.)
            
            FileBasedCollection entries = new FileBasedCollection(
                "Entries", workspaceHandle, "entries", "entry", baseDir, "entry", appURI);
            workspace.addCollection(entries);
            // want to be able to look up collection by singular and plural names
            collectionMap.put(workspaceHandle + "|entry", entries);
            collectionMap.put(workspaceHandle + "|entries", entries);

            FileBasedCollection resources = new FileBasedCollection(
                "Resources", workspaceHandle, "resources", "resource", baseDir, "*/*", appURI);
            // want to be able to look up collection by singular and plural names
            workspace.addCollection(resources);
            collectionMap.put(workspaceHandle + "|resource", resources);
            collectionMap.put(workspaceHandle + "|resources", resources);   
        }
        
        getWorkspaces().add(workspace);
    }
    
    /**
     * Find workspace by handle, returns null of not found.
     */
    FileBasedWorkspace findWorkspaceByHandle(String handle) {
        return (FileBasedWorkspace)workspaceMap.get(handle);
    }
    
    FileBasedCollection findCollectionByHandle(String handle, String collection) {
        return (FileBasedCollection)collectionMap.get(handle+"|"+collection);
    }  
}
